// File: graph.h
//-----------------------------------------------------------------------------
// Gconvert v0.2 -- A simple tool to convert different graph formats.
// Copyright (C) 2004-2005 Pascal Pons.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//-----------------------------------------------------------------------------
// Author   : Pascal Pons 
// Email    : pons@liafa.jussieu.fr
// Web page : http://liafa.jussieu.fr/~pons/
// Location : Paris, France
// Time	    : December 2005
//-----------------------------------------------------------------------------
// see readme.txt for more details


#ifndef GRAPH_H
#define GRAPH_H

#include <iostream>
#include <string>
using namespace std;

typedef enum{ADD, MULTI, ADD_EXP, ADD_LIN} Multi_edge_policy;

typedef enum{ADJACENCY_EDGE_LIST, ADJACENCY_ARC_LIST, EDGE_LIST, DYNAMIC_EDGE_LIST, ARC_LIST, DYNAMIC_ARC_LIST, WEIGHTED_EDGE_LIST, DYNAMIC_WEIGHTED_EDGE_LIST, WEIGHTED_ARC_LIST, DYNAMIC_WEIGHTED_ARC_LIST, CLIQUE,WEIGHTED_CLIQUE} Format;	// + MATRIX, PAJEK ??

class Graph;

class Vertices_name {
private:
  void increase_hash_table_size();
  void increase_name_of_number_size();
  
  unsigned int name_of_number_size;
  char** name_of_number;    // array of the name of each vertices from 0 to nb_vertices-1
  
  unsigned int hash_table_size;
  unsigned int* number_of_name;	    // hash table in which the number of each name is stored
  inline unsigned long hash(const char* s, unsigned int index);
  
public: 
  static Graph* G;
 
  unsigned int nb_vertices;
  
  const char* name(unsigned int number);
  unsigned int number(const char* name);	// add the name if not yet stored
  
  Vertices_name();
  ~Vertices_name();
};


typedef struct{unsigned int v; float w; float t;} neighbor;

class Vertex {
public:
  static Multi_edge_policy policy;  //

  static float output_time;	    // the output time if policy ADD
  static float caracteristic_time;  // the caracteristic life time of edges
  
private:
  unsigned int size;	// size of the hash table
  neighbor* H;		// the hash table

  void increase_size();
//  inline unsigned long hash(unsigned int n, int index);
public:
  unsigned int degree;

  neighbor* list();
  neighbor& operator[](unsigned int n);

  Vertex();
  ~Vertex();
};

class Graph {
private:
  void merge_components(int i, int j);
public:
  Format input_format;
  Format output_format;
  bool output_real_name;
  Multi_edge_policy multi_edge_policy;	// policy to know the weight of a multi edge
  float output_time;
  float caracteristic_time;
  string delimiters;
  bool quiet;

  int* components;
  
  void add_edge(unsigned int v1, unsigned int v2, float w, float t);
  void Graph::add_arc(unsigned int v1, unsigned int v2, float w, float t);
  
  unsigned int nb_vertices;	// the number of vertices  
  Vertex* vertices;	// array of the vertices, each vertex contains the neighbors vertices and the weight of the corresponding edge

  Vertices_name names;
  
  Graph(Format input_format, Format output_format, bool output_real_name, Multi_edge_policy multi_edge_policy, float output_time, float caracteristic_time, const char* delimiters, bool quiet);	// create an empty graph
  ~Graph();

  void compute_components();
};



ostream& operator<<(ostream& out, Graph& G);	// print the graph
istream& operator>>(istream& in, Graph& G);	// get a graph from a stream

ostream& operator<<(ostream& out, Vertices_name& names);    // print the index
#endif
